karma-jasmine-diff-print
===

> Extension for [karma-jasmine](https://github.com/karma-runner/karma-jasmine) to highlight diffs of failed equality expectations in console.

Jasmine matchers that will be processed:

- `expect(a).toEqual(b);`
- `expect(spy).toHaveBeenCalledWith(b);`
- `expect(foo).toThrow();`
- `expect(foo).toThrowError(ReferenceError, 'foo');`

Example: ![Example](http://i.imgur.com/Wrcfw4t.png "Example")

Expecatations have red background, actual results - green.

### Support

Only Jasmine 2.x is supported, this extension **will not work** with Jasmine 1.3.

### Installation

Make sure that you have [karma-jasmine](https://github.com/karma-runner/karma-jasmine) installed.

To install `karma-jasmine-diff-print` you can specify it as a devDependency in your `package.json`:

```json
{
  "devDependencies": {
    "karma": "~0.12.0",
    "karma-jasmine": "~0.2.0",
    "karma-jasmine-diff-print": "~0.1.0"
  }
}
```

or install via console:

```bash
npm install karma-jasmine-diff-print --save-dev
```

### Configuration

```js
// karma.conf.js
module.exports = function(config) {
  config.set({
    frameworks: ['jasmine', 'jasmine-diff-print'],

    files: [
      '*.js'
    ]
  });
};
```

### Dependencies

- [jsdiff](https://github.com/kpdecker/jsdiff) - JavaScript lib for text differencing

### Implementation details

The idea is to replace Jasmine core `Spec.prototype.addExpectationResult` method ([source](https://github.com/jasmine/jasmine/blob/v2.3.4/lib/jasmine-core/jasmine.js#L323)), which when called gives an opportunity to know what matcher has failed and get an error message of this failure. The following matchers are being processed:

 - toEqual
 - toHaveBeenCalledWith
 - toThrow
 - toThrowError

Only messages which contain two objects to compare will be replaced. To be more correct, they are not actual objects, but their stringified versions - Jasmine stringifies them internally. These two strings are then compared with the help of [jsdiff](https://github.com/kpdecker/jsdiff) library and inserted back into the orignal error message.

### Pitfalls

Diffs won't be displayed for a deep nested objects or large arrays, a threshold for these situations is configured in Jasmine. By default it has object nest level `MAX_PRETTY_PRINT_DEPTH = 40` and array length `MAX_PRETTY_PRINT_ARRAY_LENGTH = 100`. It means that if the diff is out of these bounds, then Jasmine will return the same strings for both compared objects and `karma-jasmine-diff-print` won't be able to highlight those diffs.

Also, because of implementation details, there is an extra stack trace message in the error stack trace, which goes very first in the list.