'use strict';

var path = require('path');

function createPattern(patternPath) {
  return { pattern: patternPath, included: true, served: true, watched: false };
}

function JasmineDiffPrint(files, logger) {
  var log = logger.create('jasmine-diff-print');

  var jasminePath = path.dirname(require.resolve('jasmine-core'));
  var coreFile = '/jasmine-core/jasmine.js';
  var jasmineCorePath = jasminePath + coreFile;

  var index = -1;
  for (var i = 0, l = files.length; i < l; i++) {
    if (files[i].pattern === jasmineCorePath) {
      index = i;
      break;
    }
  }

  if (index === -1) {
    log.warn('File "%s" not found in module "jasmine-core".', coreFile);
    log.warn('You may be using a not supported Jasmine version.');
    return false;
  }

  // inject "jasmine-diff-print" right after jasmine core but before
  // "karma-jasmine"'s adapter.js and boot so "jasmine-diff-print" could modify
  // jasmine core before it will be loaded
  // "jasmine-diff-print" requires "diff" lib, load it first

  var diffPath = require.resolve('diff');
  var jasmineDiffPrintPath = __dirname + '/src/jasmine-diff-print.js';

  files.splice(index + 1, 0, createPattern(diffPath));
  files.splice(index + 2, 0, createPattern(jasmineDiffPrintPath));
}

JasmineDiffPrint.$inject = ['config.files', 'logger'];

module.exports = {
  'framework:jasmine-diff-print': ['factory', JasmineDiffPrint]
};